@ECHO OFF
REM **************************************************************************
REM * Prepare the environment to run the Service Profiler.
REM **************************************************************************
REM * Wrightia, S.A. (c) 2006-2016
REM *
REM * Input parameters:
REM *    %1          - JVM_DIR      (The JVM installation directory)
REM *    %2          - IS_DIR       (The Integration Server installation directory)
REM *    %3          - IS_SERVICE   (Running As NT Service option)
REM *                   If running as service, no output can be sent to stdout
REM *                   under penalty of having the BATch terminate abruptly...
REM *                   ... and silently.
REM *    [%4]        - Optional.
REM *                   Contains VOID when the Core Libraries are being
REM *                   intentionally prevented from loading.
REM *                   The call is made on this situation only to log the
REM *                   occurence.
REM *
REM * Results:
REM *    The script creates 3 (three) environment variables to be used by the
REM *    caller:
REM *
REM *    ISSPROF_JVM - Options to be added to the JVM command line
REM *    ISSPROF_CP  - Path entries to the added to the -cp (CLASSPATH) of the
REM *                  JVM command line
REM *    ISSPROF_LIB - Path to the folder/directory where the needed DLLs are
REM *                  installed. This must be addded to the PATH variable,
REM *                  or other setting that can be used by the runtime to
REM *                  locate and use DLLs.
REM *
REM **************************************************************************

SET ARG_1=%~1
SET ARG_2=%~2

IF "%ARG_1%" == "/?" GOTO USAGE_INFO
IF "%ARG_1%" == "/h" GOTO USAGE_INFO
IF "%ARG_1%" == "/H" GOTO USAGE_INFO
IF "%ARG_1%" == "?"  GOTO USAGE_INFO

IF "%ARG_2%" == "" (
   ECHO.
   ECHO ERROR: Mandatory arguments are missing.
   Call :USAGE_INFO
   GOTO :ABORT_DIRECT
)

IF "1%3" == "1-service" (
   SET IS_SERVICE=Y
) ELSE (
   SET IS_SERVICE=N
)

REM // The "issprof_rsc" profiler resources directory
SET XISSPROF_RESDIR=%ARG_2%\serviceprofiler

REM // Log file name
IF NOT EXIST "%XISSPROF_RESDIR%\logs"    MD "%XISSPROF_RESDIR%\logs"
SET XISSPROF_LOGFILENAME="%XISSPROF_RESDIR%\logs\issprof.log"

Call :LOG_BEGIN "Preparing for Wrightia Service Profiler..."
Call :LOG "INFO: Integration Server = '%ARG_2%'"
IF "1-%4" == "1-VOID" (
   Call :LOG "INFO: Core libraries intentionally prevented from loading"
   GOTO BUILD_WRAPPER
)

REM // Directory to use for storing the generated ClassLoader
SET XISSPROF_TMPCLDIR=%XISSPROF_RESDIR%\tmp
IF NOT EXIST "%XISSPROF_TMPCLDIR%"       MD "%XISSPROF_TMPCLDIR%"

REM // Native lib directory
SET ISSPROF_LIB=%XISSPROF_RESDIR%\lib

REM // The interceptor classes ClassPath element
SET XISSPROF_INTCP=%ISSPROF_LIB%\wia-profiler-intc-is-11.jar

REM // Path to "java.exe"
SET XISSPROF_JAVA="%ARG_1%\bin\java.exe"

IF NOT EXIST %XISSPROF_JAVA% (
   Call :LOG "ERROR: java.exe not found. Java is NOT properly installed at:"
   Call :LOG "       '%ARG_1%'"
   GOTO :ABORT
)

Call :LOG "INFO: Java Directory = '%ARG_1%'"


SET XISSPROF_TMPCLJAR=%XISSPROF_TMPCLDIR%\issprof-cloader.jar
Call :LOG "INFO: Removing any previously generated ClassLoader..."
DEL "%XISSPROF_TMPCLJAR%" 2> nul

REM // Generating new ClassLoader
Call :LOG "INFO: Generating new ClassLoader (%XISSPROF_TMPCLJAR%)..."
%XISSPROF_JAVA% -Dwm.server.init="%ARG_2%\bin\ini.cnf" -cp "%XISSPROF_INTCP%" com.wrightia.wm.is.profiler.intercept.IssprofJarBuilder "%XISSPROF_TMPCLJAR%"

IF NOT "%ERRORLEVEL%"=="0" (
   Call :LOG "ERROR: Error generating ClassLoader (%XISSPROF_TMPCLJAR%)"
   GOTO :ABORT
)

Call :LOG "INFO: ClassLoader generated successfuly"

REM // JVM Boot (-X...) arguments
SET XISSPROF_ARGX=-Xbootclasspath/p:"%XISSPROF_TMPCLJAR%;%XISSPROF_INTCP%"
SET XISSPROF_ARGX_WIN=-Xbootclasspath/p:\"%XISSPROF_TMPCLJAR%;%XISSPROF_INTCP%\"

REM // Additional profiling properties can be set here
SET XISSPROF_PROPS=

REM // Set output parameters
SET ISSPROF_CP=
SET ISSPROF_JVM=%XISSPROF_ARGX% %XISSPROF_PROPS%
SET ISSPROF_JVM_WIN=%XISSPROF_ARGX_WIN% %XISSPROF_PROPS%


:BUILD_WRAPPER
REM // Generating IS profile wrapper configuration
If Exist %ARG_2%\..\..\..\profiles Goto :DO_WRAPPING

If Not Exist %ARG_2%\..\profiles (
   Call :LOG "INFO: This IS version does not use profiles"
   GoTo :WRAPPER_DONE
)


:DO_WRAPPING
Set XISSPROF_WRPCONF_FILENAME=wrapper.conf
Set XISSPROF_WRPCONF_UPD_FILENAME=custom_wrapper.conf
Set XISSPROF_WRPCONF_SOURCE=%ARG_2%\..\profiles\IS\configuration\%XISSPROF_WRPCONF_FILENAME%
Set XISSPROF_WRPCONF_UPDATE=%ARG_2%\..\profiles\IS\configuration\%XISSPROF_WRPCONF_UPD_FILENAME%
Echo %ARG_2% | Find "instances"
If "%ERRORLEVEL%"=="0" (
   Call :IS_INSTANCE_NAME %ARG_2%
)

If Not Exist "%XISSPROF_RESDIR%\var" MkDir "%XISSPROF_RESDIR%\var"

Set XISSPROF_WRPCONF_TARGET=%XISSPROF_RESDIR%\var\%XISSPROF_WRPCONF_FILENAME%

IF "1-%4" == "1-VOID" (
   GoTo :VOID_WRAPPER
)

If Not Exist "%XISSPROF_WRPCONF_SOURCE%" (
   Call :LOG "ERROR: File '%XISSPROF_WRPCONF_SOURCE%' not found"
   GoTo :WRAPPER_DONE
)
If Not Exist "%XISSPROF_WRPCONF_UPDATE%" (
   Call :LOG "ERROR: File '%XISSPROF_WRPCONF_UPDATE%' not found"
   GoTo :WRAPPER_DONE
)

Call :LOG "INFO: Generating IS profile wrapper configuration (%XISSPROF_WRPCONF_TARGET%)..."
%XISSPROF_JAVA% -Dwm.server.init="%ARG_2%\bin\ini.cnf" -cp "%XISSPROF_INTCP%" com.wrightia.wm.is.config.WrapperBuilder "%ISSPROF_LIB%" "%ISSPROF_JVM_WIN%" "%XISSPROF_WRPCONF_SOURCE%" "%XISSPROF_WRPCONF_TARGET%"
IF NOT "%ERRORLEVEL%"=="0" (
   Call :LOG "ERROR: Error generating IS profile wrapper configuration (%XISSPROF_WRPCONF_TARGET%)"
   Call :WRAPPER_CLEANUP
   GoTo :ABORT
)
Call :LOG "INFO: IS profile wrapper configuration file generated successfuly"

Find "%XISSPROF_WRPCONF_TARGET%" "%XISSPROF_WRPCONF_UPDATE%" > nul
If "%ERRORLEVEL%"=="0"  GoTo :WRAPPER_DONE

Copy "%XISSPROF_WRPCONF_UPDATE%" "%XISSPROF_WRPCONF_UPDATE%.beforeissprof"
Echo.                                                                      >> "%XISSPROF_WRPCONF_UPDATE%"
Echo #******************************************************************** >> "%XISSPROF_WRPCONF_UPDATE%"
Echo # Wrightia Service Profiler configuration file                        >> "%XISSPROF_WRPCONF_UPDATE%"
Echo #******************************************************************** >> "%XISSPROF_WRPCONF_UPDATE%"
Echo #include %XISSPROF_WRPCONF_TARGET%                                    >> "%XISSPROF_WRPCONF_UPDATE%"
GoTo :WRAPPER_DONE


:VOID_WRAPPER
REM // create and empty profile wrapper file
IF "1-%4" == "1-VOID" (
   Echo #encoding=UTF-8                                                     > "%XISSPROF_WRPCONF_TARGET%"
)


:WRAPPER_DONE
Call :WRAPPER_CLEANUP

IF "1-%4" == "1-VOID" (
   GOTO ABORT_DIRECT
)

GOTO RETURN


:WRAPPER_CLEANUP
   Rem cleanup wrapper specific
   Set XISSPROF_WRPCONF_FILENAME=
   Set XISSPROF_WRPCONF_UPD_FILENAME=
   Set XISSPROF_WRPCONF_SOURCE=
   Set XISSPROF_WRPCONF_TARGET=
   Set XISSPROF_WRPCONF_UPDATE=
   EXIT /B 0


:USAGE_INFO
   ECHO.
   ECHO Wrightia Service Profiler (c) 2006-2016
   ECHO.
   ECHO Usage:
   ECHO     prepare.bat JAVA_DIR IS_DIR
   ECHO where:
   ECHO     JAVA_DIR    - Java installation directory
   ECHO     IS_DIR      - Integration Server installation directory
   ECHO.
   REM // Return with ERRORLEVEL!=0 when AOP was NOT correctly prepared.
   EXIT /B -1


:RETURN
   Call :SHOW_RESULTS
   Call :CLEANUP
   EXIT /B 0


:SHOW_RESULTS
   IF "%IS_SERVICE%" == "N" (
      ECHO.
      ECHO Script Results
      ECHO ==================================
      ECHO    ISSPROF_LIB = '%ISSPROF_LIB%'
      ECHO    ISSPROF_CP  = '%ISSPROF_CP%'
      ECHO    ISSPROF_JVM = '%ISSPROF_JVM%'
      ECHO.
   )
   ECHO.                                     >> %XISSPROF_LOGFILENAME%
   ECHO Script Results                       >> %XISSPROF_LOGFILENAME%
   ECHO ==================================   >> %XISSPROF_LOGFILENAME%
   ECHO    ISSPROF_LIB = '%ISSPROF_LIB%'     >> %XISSPROF_LOGFILENAME%
   ECHO    ISSPROF_CP  = '%ISSPROF_CP%'      >> %XISSPROF_LOGFILENAME%
   ECHO    ISSPROF_JVM = '%ISSPROF_JVM%'     >> %XISSPROF_LOGFILENAME%
   ECHO.                                     >> %XISSPROF_LOGFILENAME%
   EXIT /B 0


:ABORT
   Call :LOG "."
   Call :LOG "The preparation for Wrightia Service Profiler has FAILED."
   IF "%IS_SERVICE%" == "N" (
      ECHO.
      ECHO For further diagnostics, please check the generated log file:
      ECHO     %XISSPROF_LOGFILENAME%
      ECHO.
   )


:ABORT_DIRECT
   Call :CLEANUP
   EXIT /B -1


:CLEANUP
   Rem Cleanup everything private to this script
   Set ARG_1=
   Set ARG_2=
   Set IS_SERVICE=
   Set XISSPROF_JAVA=
   Set XISSPROF_INTCP=
   Set XISSPROF_TMPCLJAR=
   Set XISSPROF_ARGX=
   Set XISSPROF_ARGX_WIN=
   Set ISSPROF_JVM_WIN=
   Set XISSPROF_PROPS=
   Set XISSPROF_LOGFILENAME=
   Set XISSPROF_RESDIR=
   Set XISSPROF_TMPCLDIR=
   EXIT /B 0


:LOG
   REM // Writes a new message both for the console and the log file
   IF "%IS_SERVICE%" == "N"  ECHO %~1

   ECHO %~1 >> %XISSPROF_LOGFILENAME%
   EXIT /b 0


:LOG_BEGIN
   REM // (re)initializes the log file
   IF "%IS_SERVICE%" == "N"  ECHO %~1

   ECHO %~1 > %XISSPROF_LOGFILENAME%
   ECHO ========================================================================= >> %XISSPROF_LOGFILENAME%
   DATE /T >> %XISSPROF_LOGFILENAME%
   TIME /T >> %XISSPROF_LOGFILENAME%
   ECHO ========================================================================= >> %XISSPROF_LOGFILENAME%
   EXIT /b 0


:IS_INSTANCE_NAME
   Set xINSTANCE_NAME=%~n1
   Call :LOG "INFO: IS instance '%xINSTANCE_NAME%'"
   Set XISSPROF_WRPCONF_SOURCE=%ARG_2%\..\..\..\profiles\IS_%xINSTANCE_NAME%\configuration\%XISSPROF_WRPCONF_FILENAME%
   Set XISSPROF_WRPCONF_UPDATE=%ARG_2%\..\..\..\profiles\IS_%xINSTANCE_NAME%\configuration\%XISSPROF_WRPCONF_UPD_FILENAME%
   Exit /b 0

